#include <util/delay.h>
#include <MVFDPanel_16S8D.h>
#include <IRremote.h>
#include <IRremoteInt.h>

#include "Board.h"

#define MAIN_LOOP_DELAY 150

uint8_t sysState;
uint8_t last_sysState;
uint8_t waitCntr;
uint8_t redValue;
uint8_t greenValue;
uint8_t blueValue;
IRrecv irrecv(VFD_IR_REC_PIN);
decode_results results;

enum enum_SysState
{
    sysSelfTest,
    sysDispScroll,
    sysDispWait,
    sysDispParams,
};

MVFD_16S8D vfd(VFD_CS_PIN, VFD_SCLK_PIN, VFD_DATAI_PIN, VFD_SHDN_PIN);    //VFD display

const char SCROLLING_DEMO[] PROGMEM = "        SCROLLING DEMO";

void setup()
{
    Serial.begin(57600);  
  
    vfd.initLED(VFD_R_PIN, VFD_G_PIN, VFD_B_PIN);
    vfd.initLightSensor(VFD_LIGHT_SNR_PIN);

    irrecv.enableIRIn();
  
    vfd.standby(false);

    redValue = 0;
    greenValue = 0;
    blueValue = 0;

    vfd.setLED(redValue, greenValue, blueValue);
  
    pinMode(VFD_DATAO_PIN, INPUT);
    vfd.reset();

    tone(VFD_SPRK_PIN, 2048, 500);
  
    vfd.setB_LED(255);  
    _delay_ms(100);
    vfd.setB_LED(0);    
    _delay_ms(100);
  
    vfd.setB_LED(255);  
    _delay_ms(100);
    vfd.setB_LED(0);    
    _delay_ms(500);

    vfd.setR_LED(255);    
    _delay_ms(100);
    vfd.setR_LED(0);      
    _delay_ms(100);
  
    vfd.setG_LED(255);      
    _delay_ms(100);
    vfd.setG_LED(0);
  
    sysState = sysSelfTest;
}

void vfd_print_f(char* str, uint8_t idx)
{
  char c;
  if(!str) return;
  while (c = str[0])
  {
    vfd.write_f_char(c, idx, false);
    idx += VFD_BYTES_PER_DIGIT;
    str++;
  }
}

void updateLEDs()
{
    blueValue++;
    if (blueValue > 60)
    {
      blueValue = 0;  
    }
    if (blueValue < 30)
    {
      vfd.setLED(blueValue/1.17, blueValue/8.6, blueValue);
    }else
    {
      uint8_t newVal = (30 - (blueValue - 30));
      vfd.setLED(newVal/1.17, newVal/8.6, newVal);
    }
}

void loop()
{  
    uint16_t lightSen = 300;
    uint8_t lightConditions = true;
    uint8_t loopCntr = MAIN_LOOP_DELAY / 10;
    char num_buf[8];
    
    while (1)
    {
        loopCntr++;
        updateLEDs();
        
        if (loopCntr >= 10)
        {
            loopCntr = 0;
            
            switch (sysState)
            {
                case sysSelfTest:
                    {
                        if (vfd.testStep() == vfd.COMPLETED)
                        {
                            sysState = sysDispScroll;
                            redValue = 0;
                            greenValue = 1;
                            vfd.initScroll_p(SCROLLING_DEMO);
                        }
                        break;
                    }

                case sysDispWait:
                    {
                        waitCntr--;
                        if (waitCntr == 0)
                          sysState = last_sysState;
                        break;
                    }

                case sysDispScroll:
                    {
                        if (!vfd.scrollStep())
                        {
                          sysState = sysDispParams;
                        }

                        break;
                    }
                    
                case sysDispParams:
                    {
                        vfd.setCur(0);
                        
                        if (lightConditions)
                          vfd.print_f_p((const char*)F("DAY "));
                        else
                          vfd.print_f_p((const char*)F("NGHT "));

                        lightSen = vfd.getLightSensorVal();
                        itoa(lightSen, num_buf, 10);

                        vfd_print_f((char*)&num_buf, 5 * VFD_BYTES_PER_DIGIT);
                        
                        vfd.flipFrame();

                        break;
                    }
                default:
                    ;       
            }
            
            //Handling RC commands
            decode_results rc_code;
            if (irrecv.decode(&rc_code))
            {
                vfd.setLED(redValue, greenValue, 255);

                vfd.setCur(0);
                if (rc_code.decode_type == NEC)
                {
                    vfd.print(F("NEC  "));
                }else if  (rc_code.decode_type == SONY)
                {
                    vfd.print(F("SONY "));
                }else if (rc_code.decode_type == RC5)
                {
                    vfd.print(F("RC5  "));
                }else if (rc_code.decode_type == RC6)
                {
                    vfd.print(F("RC6  "));
                }else 
                    vfd.print(F("UNKN "));

                vfd.print((int)rc_code.value, DEC);

                irrecv.resume();

                if (sysState != sysDispWait)
                  last_sysState = sysState;
                sysState = sysDispWait;
                waitCntr = 10;
            }
            
            if (lightSen < 200) 
            {
                //Dark conditions
                if (lightConditions == true)
                {
                    Serial.println(F("Night mode"));
                    vfd.displayOnCmd(2);
                    lightConditions = false;
                }

            }else if (lightSen > 300)
            {
                //Light conditions
                if (lightConditions == false)
                {
                    Serial.println(F("Day mode"));
                    vfd.displayOnCmd(7);
                    lightConditions = true;
                }
            }
        }
        _delay_ms(MAIN_LOOP_DELAY / 10);
    }
}

